@@ -1,15 +1,16 @@
 
 import math, time
 import shelve
-import pathlib
 import os
-
-
+from raepy.utils.singleton import Singleton
+from ..servo.servo import Servo
+from raepy.servo.servo import mutex
 
 # Distance from finger joint to midline = 0.035
 class RadialGripper(object):
-    def __init__(self,mutex,Servo):
-        self._servo = Servo
+    __metaclass__ = Singleton
+    def __init__(self):
+        self._servo = Servo()
         self._shelfdir = os.path.abspath(__file__ + "/../../") + "/shelf"
         self._mutex = mutex
         self._init_shelf()
@@ -19,24 +20,19 @@ class RadialGripper(object):
         self._mutex.acquire()
         shelf = shelve.open(self._shelfdir)
         if "fingerlength" in shelf:
-            pass
+            self._fingerlength = shelf["fingerlength"]
         else:
             shelf["fingerlength"] = 0.1
         
-        self._fingerlength =  shelf["fingerlength"]
-        
         if "radialgripper_zero_offset" in shelf:
-            pass
+            self._zero_offset = shelf["radialgripper_zero_offset"]
         else:
             shelf["radialgripper_zero_offset"] = 50
-
-        self._zero_offset = shelf["radialgripper_zero_offset"]
-
+        
         if "radialgripper_mid_axis_offset" in shelf:
-            pass
+            self._mid_axis_offset = shelf["radialgripper_mid_axis_offset"]
         else:
             shelf["radialgripper_mid_axis_offset"] = 20
-        self._mid_axis_offset = shelf["radialgripper_mid_axis_offset"]
         shelf.close()
         self._mutex.release()
 
@@ -48,7 +44,7 @@ class RadialGripper(object):
         shelf.close()
         print("raepy : fingerlength set to {} m".format(length))
 
-    def to(self, position, feedback_cb = None):
+    def to(self, position, cb=None, current=1000):
         """
         30 = i * 10 
         i gear ratio
@@ -56,28 +52,30 @@ class RadialGripper(object):
         """
         self._servo.limp()
         goal_angle = 30 * (math.degrees(math.asin(((position/2)-0.035)/self._fingerlength)) + self._mid_axis_offset)
-        self._servo.move_absolute_angle(int(goal_angle)+self._zero_offset, current=1000)
+        self._servo.move_absolute_angle(int(goal_angle)+self._zero_offset, current=current, cb=cb)
+     
 
-    def grasp(self, speed=360, current = 900, feedback_cb = None):
+    def grasp(self, speed=360, current=900, feedback_cb = None):
         self._servo.jog(-speed, current=current)
         time.sleep(0.02)
         while self._servo.get_motor_mode() != 'Holding':
+            if feedback_cb:
+                feedback_cb(self.fingertip_distance(), self._servo.actual_current())
             time.sleep(0.01)
 
     def openfingers(self):
         self.to(0.2)
 
     def release(self, feedback_cb = None):
-        self._servo.move_relative_angle(400)
-        time.sleep(1)
+        self._servo.move_relative_angle(800)
         self._servo.limp()
 
-    def calibrate(self, feedback_cb = None):
+    def calibrate(self, current=700,feedback_cb = None):
         if self._fingerlength == None:
             print("raepy: set fingerlength before calling calibrating the gripper")
             return
         self._servo.jog(-50)
-        while self._servo.actual_current() < 550:
+        while self._servo.actual_current() < current:
             time.sleep(0.01)
         self._servo.limp()
         time.sleep(1)
@@ -92,6 +90,7 @@ class RadialGripper(object):
         shelf.close()
         self._mutex.release()
         print("raepy : offset angle set to {} deg".format(self._mid_axis_offset))
+        self._servo.limp()
 
     def fingertip_distance(self):
         """
