@@ -1,37 +1,51 @@
-from typing import Type
 import time
 
-import pathlib
 from raepy.servo.LSS_Library_Python import lss
-
 from raepy.servo.LSS_Library_Python import lssc
 
+from serial.serialutil import SerialException
+
 from .exceptions import SerialConnectionError
 
-from serial.tools import list_ports
+import sys
+import glob
+import serial
 import os
-from pathlib import Path
 
 import shelve
 
+from raepy.utils.singleton import Singleton
+
+from threading import Lock
+
+mutex = Lock()
 
 
 class Servo(object):
-    def __init__(self, mutex = None,CST_LSS_Port="/dev/ttyS0", CST_LSS_Baud = lssc.LSS_DefaultBaud):
-        self._CST_LSS_Port = CST_LSS_Port
+    __metaclass__ = Singleton
+    def __init__(self, CST_LSS_Baud = lssc.LSS_DefaultBaud):
+
         self._CST_LSS_Baud = CST_LSS_Baud
-        lss.initBus(self._CST_LSS_Port,self._CST_LSS_Baud)
-        self._lss = lss.LSS(0)
+        serial_ports_list = self.serial_ports()
+
+        for port in serial_ports_list:
+            try:
+                self.init_interface(port)
+            except SerialException:
+                print("SerialException on port {}".format(port))
+            if self.get_target_angle() != None:
+                break
+
+
         self._shelfdir = os.path.abspath(__file__ + "/../../") +"/shelf"
         self._mutex = mutex
         self._mutex.acquire()
         shelf = shelve.open(self._shelfdir)
         if "Servo_offset" in shelf:
-            pass
+            self._offset_angle = shelf["Servo_offset"]
         else:
             shelf["Servo_offset"] = 0
-            
-        self._offset_angle = shelf["Servo_offset"]
+            self._offset_angle = 0
         shelf.close()
         self._mutex.release()
 
@@ -61,6 +75,38 @@ class Servo(object):
             10:'Safe Mode'
         }
 
+    def init_interface(self, dev_string):
+        lss.initBus(dev_string,self._CST_LSS_Baud)
+        self._lss = lss.LSS(0)
+
+    def serial_ports(self):
+        """ Lists serial port names
+
+            :raises EnvironmentError:
+                On unsupported or unknown platforms
+            :returns:
+                A list of the serial ports available on the system
+        """
+        if sys.platform.startswith('win'):
+            ports = ['COM%s' % (i + 1) for i in range(256)]
+        elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
+            # this excludes your current terminal "/dev/tty"
+            ports = glob.glob('/dev/tty[A-Za-z]*')
+        elif sys.platform.startswith('darwin'):
+            ports = glob.glob('/dev/tty.*')
+        else:
+            raise EnvironmentError('Unsupported platform')
+
+        result = []
+        for port in ports:
+            try:
+                s = serial.Serial(port)
+                s.close()
+                result.append(port)
+            except (OSError, serial.SerialException):
+                pass
+        return result
+
 
     def jog(self,speed, current=700):
         """
@@ -75,17 +121,19 @@ class Servo(object):
             if cnt > 5:
                 raise SerialConnectionError
 
-    def move_absolute_angle(self, angle, current=700):
+    def move_absolute_angle(self, angle, current=700,cb=None):
         """
         Absolute Positioning in 1/10 degree: 500 => 50°
         angle: target angle in 1/10 °
         current: threshold where motor halt and holds
         """
-
-        if abs(self.actual_angle() - angle) < 5:
+    
+        if abs(self.actual_angle() - angle - self._offset_angle) < 5:
             return
 
-        self._lss.move(int(angle) - self._offset_angle , current)
+        initial_angle = self.actual_angle()
+
+        self._lss.move(int(angle) - self._offset_angle, current)
 
         cnt = 0
         while self.get_target_angle() - self._offset_angle != angle:
@@ -97,6 +145,8 @@ class Servo(object):
 
         while abs(self.get_target_angle() - self._offset_angle - self.actual_angle()) > 10:
             self._lss.move(int(angle) + self._offset_angle, current)
+            if cb:
+                cb(initial_angle, angle - self._offset_angle, self.actual_angle())
             time.sleep(0.02)
 
         
@@ -215,7 +265,7 @@ class Servo(object):
 
         return True
 
-    def reset_motor(self):
+    def reset(self):
         """
         Reset Motor
         """
